/*
	in_sdl.c
	SDL game input code

	$Id: in_sdl.c,v 1.50 2007/09/21 11:05:08 sezero Exp $
*/

#include "sdl_inc.h"
#include "quakedef.h"
#include "minimal.h"
#include "vid.h"
#include "keys.h"

// mouse variables
static cvar_t	m_filter = {"m_filter", "0", CVAR_NONE};
static int	mouse_x, mouse_y, old_mouse_x, old_mouse_y;
static qboolean	mouseactive = false;
static qboolean	mouseinitialized = false;
static qboolean	mouseactivatetoggle = false;
static qboolean	mouseshowtoggle = true;

static int buttonremap[] = {
	K_MOUSE1,
	K_MOUSE3,	/* right button		*/
	K_MOUSE2,	/* middle button	*/
	K_MWHEELUP,
	K_MWHEELDOWN,
	K_MOUSE4,
	K_MOUSE5
    };

/*===========
Force_CenterView_f
===========*/
static void Force_CenterView_f (void){
	cl.viewangles[PITCH] = 0;
    }

/*===========
IN_ShowMouse
===========*/
void IN_ShowMouse (void){
	// no need to check mouseinitialized here
	if (!mouseshowtoggle){
		SDL_ShowCursor(0);
		gp2x_video_cursor_show(0);
		mouseshowtoggle = true;
        }
    }

/*===========
IN_HideMouse
===========*/
void IN_HideMouse (void){
	// no need to check mouseinitialized here
	if (mouseshowtoggle){
		SDL_ShowCursor (0);
		gp2x_video_cursor_show(0);
		mouseshowtoggle = false;
        }
    }


/* ============================================================
   NOTES on enabling-disabling the mouse:
   - In windowed mode, mouse is temporarily disabled in main
     menu, so the un-grabbed pointer can be used for desktop
     This state is stored in menu_disabled_mouse as true
   - In fullscreen mode, we don't disable the mouse in menus,
     if we toggle windowed/fullscreen, the above state variable
     is used to correct this in VID_ToggleFullscreen()
   - In the console mode and in the options menu-group, mouse
     is not disabled, and menu_disabled_mouse is set to false
   - Starting a or connecting to a server activates the mouse
     and sets menu_disabled_mouse to false
   - Pausing the game disables (so un-grabs) the mouse, unpausing
     activates it. We don't play with menu_disabled_mouse in
     such cases
*/

/*===========
IN_ActivateMouse
===========*/
void IN_ActivateMouse (void){
	if (!mouseinitialized){return;}
	if (!mouseactivatetoggle){
		if (_enable_mouse.integer){
			mouseactivatetoggle = true;
			mouseactive = true;
			SDL_WM_GrabInput (SDL_GRAB_ON);
            }
        }
    }

/*
===========
IN_DeactivateMouse
===========
*/
void IN_DeactivateMouse (void){
	if (!mouseinitialized){return;}
	if (mouseactivatetoggle){
		mouseactivatetoggle = false;
		mouseactive = false;
		SDL_WM_GrabInput (SDL_GRAB_OFF);
        }
    }

/*
===========
IN_StartupMouse
===========
*/
static void IN_StartupMouse (void){
	IN_HideMouse();
	if (safemode || COM_CheckParm ("-nomouse")){
		SDL_WM_GrabInput (SDL_GRAB_OFF);
		return;
        }

	mouseinitialized = true;
	if (_enable_mouse.integer){IN_ActivateMouse ();}
}

/*===========
IN_Init
===========*/
void IN_Init (void){
	// mouse variables
	Cvar_RegisterVariable (&m_filter);
	Cmd_AddCommand ("force_centerview", Force_CenterView_f);
	IN_StartupMouse ();
	SDL_EnableUNICODE (1);	/* needed for input in console */
	SDL_EnableKeyRepeat (SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL*2);
    }

/*===========
IN_Shutdown
===========*/
void IN_Shutdown (void){
	IN_DeactivateMouse ();
	IN_ShowMouse ();
	mouseinitialized = false;
    }

/*===========
IN_ReInit
===========*/
void IN_ReInit (void){
	IN_StartupMouse ();
	SDL_EnableUNICODE (1);	/* needed for input in console */
	SDL_EnableKeyRepeat (SDL_DEFAULT_REPEAT_DELAY, SDL_DEFAULT_REPEAT_INTERVAL*2);
    }

/*===========
IN_MouseMove
===========*/
static void IN_MouseMove (usercmd_t *cmd){
	int		mx, my;
	SDL_GetRelativeMouseState(&mx,&my);
	if (m_filter.integer){
		mouse_x = (mx + old_mouse_x) * 0.5;
		mouse_y = (my + old_mouse_y) * 0.5;
        }
	else{
		mouse_x = mx;
		mouse_y = my;
        }

	old_mouse_x = mx;
	old_mouse_y = my;
	mouse_x *= sensitivity.value;
	mouse_y *= sensitivity.value;

    // add mouse X/Y movement to cmd
	if ( (in_strafe.state & 1) || (lookstrafe.integer && (in_mlook.state & 1) )){
		cmd->sidemove += m_side.value * mouse_x;
        }
	else{cl.viewangles[YAW] -= m_yaw.value * mouse_x;}

	if (in_mlook.state & 1){
		if (mx || my){V_StopPitchDrift ();}
        }

	if ( (in_mlook.state & 1) && !(in_strafe.state & 1)){
		cl.viewangles[PITCH] += m_pitch.value * mouse_y;
		if (cl.viewangles[PITCH] > 80) {cl.viewangles[PITCH] = 80;}
		if (cl.viewangles[PITCH] < -70) {cl.viewangles[PITCH] = -70;}
        }
	else{
		if ((in_strafe.state & 1) && (cl.v.movetype == MOVETYPE_NOCLIP)){
			cmd->upmove -= m_forward.value * mouse_y;
            }
		else{cmd->forwardmove -= m_forward.value * mouse_y;}
        }

    // Did keyboard set it already??
	if (cl.idealroll == 0){
		if (cl.v.movetype == MOVETYPE_FLY){
			if (mouse_x < 0){cl.idealroll = -10;}
			else{
			    if (mouse_x > 0){cl.idealroll = 10;}
                }
            }
        }
    }

/*===========
IN_Move
===========*/
void IN_Move (usercmd_t *cmd){
	if (cl.v.cameramode){
		memset (cmd, 0, sizeof(*cmd));
		return;
        }
	if (mouseactive){IN_MouseMove (cmd);}
    }

/*===========
IN_Commands
===========*/
void IN_Commands (void){}

/*===========
IN_SendKeyEvents
===========*/
extern keydest_t key_dest;
unsigned long   GP2X_JoystickStatePrev;
unsigned int    GP2X_Run = 0;

void IN_SendKeyEvents (void){

	 unsigned long GP2X_JoystickState = gp2x_joystick_read();

    //Game Commands
    if (key_dest == 0){

        if ((GP2X_JoystickState & GP2X_UP) && !(GP2X_JoystickStatePrev & GP2X_UP)){Cbuf_AddText("+forward\n");}
        if (!(GP2X_JoystickState & GP2X_UP) && (GP2X_JoystickStatePrev & GP2X_UP)){Cbuf_AddText("-forward\n");}
        if ((GP2X_JoystickState & GP2X_DOWN) && !(GP2X_JoystickStatePrev & GP2X_DOWN)){Cbuf_AddText("+back\n");}
        if (!(GP2X_JoystickState & GP2X_DOWN) && (GP2X_JoystickStatePrev & GP2X_DOWN)){Cbuf_AddText("-back\n");}
        if ((GP2X_JoystickState & GP2X_LEFT) && !(GP2X_JoystickStatePrev & GP2X_LEFT)){Cbuf_AddText("+left\n");}
        if (!(GP2X_JoystickState & GP2X_LEFT) && (GP2X_JoystickStatePrev & GP2X_LEFT)){Cbuf_AddText("-left\n");}
        if ((GP2X_JoystickState & GP2X_RIGHT) && !(GP2X_JoystickStatePrev & GP2X_RIGHT)){Cbuf_AddText("+right\n");}
        if (!(GP2X_JoystickState & GP2X_RIGHT) && (GP2X_JoystickStatePrev & GP2X_RIGHT)){Cbuf_AddText("-right\n");}
        //Jump:
        if ((GP2X_JoystickState & GP2X_Y) && !(GP2X_JoystickStatePrev & GP2X_Y)){Cbuf_AddText("+jump\n");}
        if (!(GP2X_JoystickState & GP2X_Y) && (GP2X_JoystickStatePrev & GP2X_Y)){Cbuf_AddText("-jump\n");}
        //Crouch:
        if ((GP2X_JoystickState & GP2X_X) && !(GP2X_JoystickStatePrev & GP2X_X)){Cbuf_AddText("+crouch\n");}
        if (!(GP2X_JoystickState & GP2X_X) && (GP2X_JoystickStatePrev & GP2X_X)){Cbuf_AddText("-crouch\n");}
        //Attack:
        if ((GP2X_JoystickState & GP2X_B) && !(GP2X_JoystickStatePrev & GP2X_B)){Cbuf_AddText("+attack\n");}
        if (!(GP2X_JoystickState & GP2X_B) && (GP2X_JoystickStatePrev & GP2X_B)){Cbuf_AddText("-attack\n");}
        //Use item:
        if ((GP2X_JoystickState & GP2X_A) && !(GP2X_JoystickStatePrev & GP2X_A)){Key_Event(K_ENTER, 1);}
        if (!(GP2X_JoystickState & GP2X_A) && (GP2X_JoystickStatePrev & GP2X_A)){Key_Event(K_ENTER, 0);}
        //Strafe Left:
        if ((GP2X_JoystickState & GP2X_L) && !(GP2X_JoystickStatePrev & GP2X_L)){Cbuf_AddText("+moveleft\n");}
        if (!(GP2X_JoystickState & GP2X_L) && (GP2X_JoystickStatePrev & GP2X_L)){Cbuf_AddText("-moveleft\n");}
        //Strafe Right:
        if ((GP2X_JoystickState & GP2X_R) && !(GP2X_JoystickStatePrev & GP2X_R)){Cbuf_AddText("+moveright\n");}
        if (!(GP2X_JoystickState & GP2X_R) && (GP2X_JoystickStatePrev & GP2X_R)){Cbuf_AddText("-moveright\n");}
        //Start Menu:
        if ((GP2X_JoystickState & GP2X_START) && !(GP2X_JoystickStatePrev & GP2X_START)){
            Cbuf_AddText("-forward\n");
            Cbuf_AddText("-back\n");
            Cbuf_AddText("-left\n");
            Cbuf_AddText("-right\n");
            Cbuf_AddText("-jump\n");
            Cbuf_AddText("-crouch\n");
            Cbuf_AddText("-attack\n");
            Key_Event(K_ENTER, 0);
            Cbuf_AddText("-moveleft\n");
            Cbuf_AddText("-moveright\n");
            M_ToggleMenu_f ();
            }
        //Toggle Run:
        if ((GP2X_JoystickState & GP2X_SELECT) && !(GP2X_JoystickStatePrev & GP2X_SELECT)){Key_Event(K_SHIFT,GP2X_Run);GP2X_Run = !GP2X_Run;}
        //Change Weapon:
        if ((GP2X_JoystickState & GP2X_VOL_DOWN) && !(GP2X_JoystickStatePrev & GP2X_VOL_DOWN)){Cbuf_AddText("impulse 10\n");}
        //Change Item:
        if ((GP2X_JoystickState & GP2X_VOL_UP) && !(GP2X_JoystickStatePrev & GP2X_VOL_UP)){Key_Event(']', 1);}
        if (!(GP2X_JoystickState & GP2X_VOL_UP) && (GP2X_JoystickStatePrev & GP2X_VOL_UP)){Key_Event(']', 0);}
        GP2X_JoystickStatePrev = GP2X_JoystickState;
        }

    //Console Commands:
    else if (key_dest == 1){
        //Test FPS:
        if ((GP2X_JoystickState & GP2X_Y) && !(GP2X_JoystickStatePrev & GP2X_Y)){Cbuf_AddText("timerefresh\n");}
        if ((GP2X_JoystickState & GP2X_X) && !(GP2X_JoystickStatePrev & GP2X_X)){Cbuf_AddText("profile\n");}

        if ((GP2X_JoystickState & GP2X_L) && !(GP2X_JoystickStatePrev & GP2X_L)){Key_Event(K_ESCAPE, 1);}
        if (!(GP2X_JoystickState & GP2X_L) && (GP2X_JoystickStatePrev & GP2X_L)){Key_Event(K_ESCAPE, 0);}
        if ((GP2X_JoystickState & GP2X_R) && !(GP2X_JoystickStatePrev & GP2X_R)){Key_Event(K_ENTER, 1);}
        if (!(GP2X_JoystickState & GP2X_R) && (GP2X_JoystickStatePrev & GP2X_R)){Key_Event(K_ENTER, 0);}
        GP2X_JoystickStatePrev = GP2X_JoystickState;
        }

    //Menu Commands:
    else {
        if ((GP2X_JoystickState & GP2X_UP) && !(GP2X_JoystickStatePrev & GP2X_UP)){Key_Event(K_UPARROW,1);}
        if (!(GP2X_JoystickState & GP2X_UP) && (GP2X_JoystickStatePrev & GP2X_UP)){Key_Event(K_UPARROW,0);}
        if ((GP2X_JoystickState & GP2X_DOWN) && !(GP2X_JoystickStatePrev & GP2X_DOWN)){Key_Event(K_DOWNARROW,1);}
        if (!(GP2X_JoystickState & GP2X_DOWN) && (GP2X_JoystickStatePrev & GP2X_DOWN)){Key_Event(K_DOWNARROW,0);}
        if ((GP2X_JoystickState & GP2X_LEFT) && !(GP2X_JoystickStatePrev & GP2X_LEFT)){Key_Event(K_LEFTARROW,1);}
        if (!(GP2X_JoystickState & GP2X_LEFT) && (GP2X_JoystickStatePrev & GP2X_LEFT)){Key_Event(K_LEFTARROW,0);}
        if ((GP2X_JoystickState & GP2X_RIGHT) && !(GP2X_JoystickStatePrev & GP2X_RIGHT)){Key_Event(K_RIGHTARROW,1);}
        if (!(GP2X_JoystickState & GP2X_RIGHT) && (GP2X_JoystickStatePrev & GP2X_RIGHT)){Key_Event(K_RIGHTARROW,0);}

        if ((GP2X_JoystickState & GP2X_L) && !(GP2X_JoystickStatePrev & GP2X_L)){Key_Event(K_ESCAPE, 1);}
        if (!(GP2X_JoystickState & GP2X_L) && (GP2X_JoystickStatePrev & GP2X_L)){Key_Event(K_ESCAPE, 0);}
        if ((GP2X_JoystickState & GP2X_R) && !(GP2X_JoystickStatePrev & GP2X_R)){Key_Event(K_ENTER, 1);}
        if (!(GP2X_JoystickState & GP2X_R) && (GP2X_JoystickStatePrev & GP2X_R)){Key_Event(K_ENTER, 0);}

        if ((GP2X_JoystickState & GP2X_X) && !(GP2X_JoystickStatePrev & GP2X_X)){Key_Event(K_ESCAPE, 1);}
        if (!(GP2X_JoystickState & GP2X_X) && (GP2X_JoystickStatePrev & GP2X_X)){Key_Event(K_ESCAPE, 0);}
        if ((GP2X_JoystickState & GP2X_B) && !(GP2X_JoystickStatePrev & GP2X_B)){Key_Event(K_ENTER, 1);}
        if (!(GP2X_JoystickState & GP2X_B) && (GP2X_JoystickStatePrev & GP2X_B)){Key_Event(K_ENTER, 0);}
        GP2X_JoystickStatePrev = GP2X_JoystickState;
        }
    }

